# -*- coding: utf-8 -*-
from functools import wraps
from flask import jsonify

from jotonce import factory
from jotonce.helpers import JSONEncoder
from schematics.exceptions import ModelValidationError


def create_app(settings_override=None, register_security_blueprint=False):
    """Returns the JotOnce API application instance.
    """

    app = factory.create_app(__name__, __path__, settings_override)

    # Set the default JSON encoder
    app.json_encoder = JSONEncoder

    # Register custom error handlers
    app.errorhandler(404)(on_404)
    app.errorhandler(ModelValidationError)(on_validation_error)

    return app


def route(bp, *args, **kwargs):
    kwargs.setdefault('strict_slashes', False)

    def decorator(f):
        @bp.route(*args, **kwargs)
        @wraps(f)
        def wrapper(*args, **kwargs):
            status_code = 200
            payload = f(*args, **kwargs)
            if isinstance(payload, tuple):
                status_code = payload[1]
                payload = payload[0]
            return jsonify(dict(data=payload)), status_code
        return f

    return decorator


def on_validation_error(e):
    return jsonify(dict(error=e.message)), 400


def on_404(e):
    return jsonify(dict(error='Not found')), 404
