# -*- coding: utf-8 -*-
import logging
from flask import request, Blueprint
from schematics.exceptions import ModelValidationError
from jotonce.api import route
from jotonce.messages.models import Message

bp = Blueprint('message', __name__, url_prefix='/message')
log = logging.getLogger(__name__)


@route(bp, '/<passphrase>')
def get(passphrase):
    log.debug("recieved args: %s to /get", request.args)
    message = Message.get_by_passphrase(passphrase)
    # TODO: Maybe store pin in an auth header? research me.
    pin = request.args.get('pin')
    if not message:
        return
    if not message.get('pin'):
        # The message doesn't require a pin number, we can return.
        return message.to_primitive()
    if not pin:
        raise ModelValidationError({"pin": "This Jot requires a pin number."})
    if pin != message.get('pin'):
        raise ModelValidationError({"pin": "Invalid pin number."})
    return message.to_primitive()


@route(bp, '/<passphrase>', methods=['POST'])
def create(passphrase):
    if Message.get_by_passphrase(passphrase):
        raise ModelValidationError('Jot already exists.')
    message = Message({
        'passphrase': passphrase,
        'message': request.json.get('message'),
        'pin': request.json.get('pin'),
        'expires_on': request.json.get('expires_on'),
        'send_read_receipt': request.json.get('send_read_receipt')
    })
    message.save()
    return {"edit_key": message.edit_key}


@route(bp, '/<passphrase>', methods=['PUT'])
def update(passphrase):
    # TODO: move this into a header field.
    edit_key = request.args.get('edit_key')
    message = Message({
        'passphrase': passphrase,
        'message': request.json.get('message'),
        'pin': request.json.get('pin'),
        'expires_on': request.json.get('expires_on'),
        'send_read_receipt': request.json.get('send_read_receipt'),
        'edit_key': edit_key
    })
    message.save(overwrite=True)
    return {"edit_key": message.edit_key}


@route(bp, '/<passphrase>', methods=['DELETE'])
def delete(passphrase):
    # TODO: move this into a header field
    edit_key = request.args.get('edit_key')
    Message.delete_by_passphrase(passphrase, edit_key)
