# -*- coding: utf-8 -*-
from flask import Flask
from jotonce.core import db
from jotonce.helpers import register_blueprints
from jotonce.middleware import HTTPMethodOverrideMiddleware


def create_app(package_name, package_path, settings_override=None):
    """Returns a :class:`Flask` application instance configured with common
    functionality for the JotOnce platform.

    :param package_name:
        application package name.
    :param package_path:
        application package path.
    :param settings_override:
        a dictionary of settings to override.
    """
    app = Flask(package_name, instance_relative_config=True)

    app.config.from_object('jotonce.settings')
    app.config.from_pyfile('settings.cfg', silent=True)
    app.config.from_object(settings_override)

    register_blueprints(app, package_name, package_path)
    app.wsgi_app = HTTPMethodOverrideMiddleware(app.wsgi_app)
    db.init_app(app)
    return app
