# -*- coding: utf-8 -*-
import datetime
import uuid

import dateutil.parser
from schematics.models import Model
from schematics.types import StringType, DateTimeType, BooleanType
from schematics.exceptions import ModelValidationError

from jotonce.core import db


class Message(Model):
    passphrase = StringType(required=True)
    message = StringType(required=True)
    created_on = DateTimeType(required=True, default=datetime.datetime.now)
    expires_on = DateTimeType(required=True)
    pin = StringType(required=False)
    send_read_receipt = BooleanType(required=True, default=False)
    # The edit key is a UUID given to the client and acts as a password
    # allowing for post-save edits to the message.
    edit_key = StringType(required=True, default=str(uuid.uuid4()))

    def save(self, overwrite=False):
        self.validate()
        jot = Message.get_by_passphrase(self.passphrase)
        if overwrite:
            if jot.edit_key != self.edit_key:
                raise ModelValidationError(
                    {"edit_key": "the edit key you provided does not match"})
        else:
            raise ModelValidationError("Jot already exists")
        with db.connection() as c:
            c.hmset('jot:' + self.passphrase, {
                'message': self.message,
                'created_on': self.created_on,
                'expires_on': self.expires_on,
                'pin': self.pin,
                'send_read_receipt': self.send_read_receipt,
                'edit_key': self.edit_key
            })
            c.expireat(self.passphrase, self.expires_on)

    @staticmethod
    def get_by_passphrase(passphrase):
        jot = None
        with db.connection() as c:
            jot = c.hgetall('jot:' + passphrase)
            if not jot:
                return jot
        # TODO: do this coercion in the model.
        jot['expires_on'] = dateutil.parser.parse(jot.get('expires_on'))
        jot['created_on'] = dateutil.parser.parse(jot.get('created_on'))
        return Message(jot)

    @staticmethod
    def delete_by_passphrase(passphrase, edit_key):
        jot = Message.get_by_passphrase(passphrase)
        if not jot:
            raise ModelValidationError("Jot does not exist")
        if jot.edit_key != edit_key:
            raise ModelValidationError(
                {"edit_key": "the edit key you provided does not match"})
        with write_db.connection() as c:
            c.delete("jot:" + passphrase)
