# -*- coding: utf-8 -*-
from datetime import datetime
import logging
from schematics.models import Model
from schematics.types import StringType
from jotonce.core import db
from jotonce.helpers import parse_timestamp


log = logging.getLogger(__name__)


class Passphrase(Model):
    passphrase = StringType(required=True)

    def save(self):
        with db.connection() as c:
            return c.sadd("sug", self.passphrase)

    @staticmethod
    def create_suggestion(passphrase):
        with db.connection() as c:
            return c.sadd("sug", passphrase)

    @staticmethod
    def mget_all_suggestions():
        with db.connection() as c:
            return c.smembers("sug")

    @staticmethod
    def mget_available_suggestions():
        pass

    @staticmethod
    def mget_unavailable_suggestions():
        pass

    @staticmethod
    def mark_phrase_as_in_use(passphrase, release_at):
        if isinstance(release_at, datetime):
            log.debug("converting release_at from datetime to seconds")
            release_at = release_at.strftime('%s')
        with db.connection() as c:
            return c.zadd("inuse", release_at, passphrase)

    @staticmethod
    def mget_all_in_use(include_released=False, with_release_times=False):
        now = datetime.now().strftime('%s')
        with db.connection() as c:
            if include_released:
                return c.zrange("inuse", start=0, end=-1,
                                withscores=with_release_times,
                                score_cast_func=parse_timestamp)
            else:
                return c.zrangebyscore("inuse", min=0, max=now,
                                       withscores=with_release_times,
                                       score_cast_func=parse_timestamp)
